"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var target_queries_1 = require("./target-queries");
var trigger_element_monitor_1 = require("./trigger-element-monitor");
var distinct_1 = require("./distinct");
function attach(widget) {
    var triggerElements = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        triggerElements[_i - 1] = arguments[_i];
    }
    widget.implementation.events.subscribe('widget:create-instance', function (e, data) {
        /**
         * Listen through the implementation instead of the widget directly, else
         * event listeners will be removed if the widget is destroyed and re-created.
         */
        if (e.target === widget) {
            var monitor = widget.container.get('bootstrapMonitor');
            if (!monitor) {
                // create and register if not available
                monitor = new trigger_element_monitor_1.default(widget);
                widget.container.register('bootstrapMonitor', monitor);
            }
            monitor.watch(triggerElements);
        }
    });
    return widget.load();
}
exports.attach = attach;
function scan(implementation, rootElement) {
    if (rootElement === void 0) { rootElement = document; }
    // scan 'rootElement' for trigger elements matching the bindings
    var bindings = implementation.bindings;
    var triggerElementMap = [].concat(target_queries_1.queryLinkTargets(rootElement, bindings), target_queries_1.queryPreRenderedTargets(rootElement, bindings));
    // iterate each unique widget name and attach it to its detected trigger elements
    var uniqueWidgetNames = distinct_1.default(triggerElementMap.map(function (_a) {
        var widgetName = _a.widgetName;
        return widgetName;
    }));
    return Promise.all(uniqueWidgetNames.map(function (widgetName) {
        var widget = implementation.widgets.get(widgetName);
        if (!widget) {
            if (process.env.NODE_ENV !== 'production') {
                // tslint:disable-next-line max-line-length
                console.warn("Webprovisions: A trigger element for \"" + widgetName + "\" was found but no widget with that name is available on the implementation (\"" + implementation.name + "\"). Make sure this is not a mistake.");
            }
            return;
        }
        var triggerElements = triggerElementMap
            .filter(function (item) { return item.widgetName === widgetName; })
            .map(function (item) { return item.element; });
        return attach.apply(void 0, [widget].concat(triggerElements));
    }));
}
exports.scan = scan;
function domContentLoaded() {
    return new Promise(function (resolve) {
        if (document.readyState === 'loading') {
            document.addEventListener('DOMContentLoaded', function () { return resolve(); });
        }
        else {
            resolve();
        }
    });
}
var createImplementationSelector = function (config) {
    return config.tenant ? config.name + ":" + config.tenant : config.name;
};
function bootstrapImplementation(config, environment, createConfigurationHandlers, rootElementOrResolver) {
    if (createConfigurationHandlers === void 0) { createConfigurationHandlers = function () { return []; }; }
    var implementationSelector = createImplementationSelector(config);
    var implementation = environment.implementations.find(implementationSelector);
    if (implementation) {
        if (process.env.NODE_ENV !== 'production') {
            // tslint:disable-next-line max-line-length
            console.error("Webprovisions: Implementation \"" + implementationSelector + "\" already exist on current environment (" + environment.name + "). Make sure the file isn't included twice.");
        }
        return Promise.resolve(implementation);
    }
    implementation = environment.createImplementation(config);
    implementation.scan = function (rootElement) {
        return scan(implementation, rootElement);
    };
    implementation.attach = function (widget) {
        var triggerElements = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            triggerElements[_i - 1] = arguments[_i];
        }
        return attach.apply(void 0, [widget].concat(triggerElements));
    };
    if (typeof environment.configure === 'undefined') {
        environment.configure = function (selector, configFn) {
            environment.widgets.query(selector).forEach(function (widget) {
                return widget.implementation.configure(configFn);
            });
        };
    }
    var rootElementResolver = typeof rootElementOrResolver === 'function'
        ? rootElementOrResolver : function () { return rootElementOrResolver; };
    return domContentLoaded()
        .then(function () {
        implementation.configure(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            createConfigurationHandlers()
                .forEach(function (configureHandler) { return configureHandler.apply(void 0, args); });
        });
    })
        .then(function () { return scan(implementation, rootElementResolver()); })
        .then(function () { return implementation; });
}
exports.default = bootstrapImplementation;
//# sourceMappingURL=bootstrap-implementation.js.map